/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.cdm;


import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.greenstone.gatherer.Configuration;
import org.greenstone.gatherer.DebugStream;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.gui.GLIButton;
import org.greenstone.gatherer.util.JarTools;
import org.greenstone.gatherer.util.XMLTools;
import org.w3c.dom.*;

/** This class manages the language commands, remembering both a list of languages to build indexes in, plus the default language.
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 */
public class LanguageManager
    extends DOMProxyListModel {

    static public Document LANGUAGES_DOCUMENT = XMLTools.parseXMLFile("xml/languages.xml", true);

    static final private Dimension COMPONENT_SIZE = new Dimension(125,25);

    /** The visual controls for this manager. */
    private Control controls = null;
    /** A reference to this class as a model, for the inner controls class. */
    private DOMProxyListModel model = null;
    /** A hashtable of code->name mappings of known languages. */
    private LinkedHashMap known_languages = null;
    /** The default language object. */
    private Language default_language = null;
 
   /** Constructor. */
    public LanguageManager(Element languages_element) {
	super(languages_element, CollectionConfiguration.LANGUAGE_ELEMENT, new Language());
	
	DebugStream.println("LanguageManager: " + getSize() + " languages parsed.");

	this.model = this;
	// Retrieve the default language
	NodeList default_language_elements = CollectionDesignManager.collect_config.getDocumentElement().getElementsByTagName(CollectionConfiguration.LANGUAGE_DEFAULT_ELEMENT);
	if(default_language_elements.getLength() > 0) {
	    default_language = new Language((Element)default_language_elements.item(0));
	}
	// Load a series of code->language mappings into known_languages, by reading from the 'languages.xml' file, which is essentially a subset of the ISO 639 Standard.
	known_languages = new LinkedHashMap();
	/*
	  try {
	    File in_file = new File("languages.dat");
	    FileReader in_reader = new FileReader(in_file);
	    BufferedReader in = new BufferedReader(in_reader);
	    String entry = null;
	    while((entry = in.readLine()) != null) {
		if(!entry.startsWith("#")) {
		    StringTokenizer tokenizer = new StringTokenizer(entry);
		    if(tokenizer.countTokens() >= 2) {
			String name = tokenizer.nextToken();
			String code = tokenizer.nextToken().toLowerCase();
			known_languages.put(code, name);
		    }
		}
	    }
	    in.close();
	    in_reader.close();
	    in = null;
	    in_reader = null;
	    in_file = null;
	}
	catch (Exception error) {
	    error.printStackTrace();
	}
	*/
	NodeList language_elements = LANGUAGES_DOCUMENT.getDocumentElement().getElementsByTagName(CollectionConfiguration.LANGUAGE_ELEMENT);
	for(int i = 0; i < language_elements.getLength(); i++) {
	    Element language_element = (Element) language_elements.item(i);
	    String code = language_element.getAttribute(CollectionConfiguration.CODE_ATTRIBUTE);
	    String name = language_element.getAttribute(CollectionConfiguration.NAME_ATTRIBUTE);
	    known_languages.put(code.toLowerCase(), name);
	    name = null;
	    code = null;
	    language_element = null;
	}
    }

    /** Method to add a new language.
     * @param language The <strong>Language</strong> to add.
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.collection.CollectionManager
     */
    private void addLanguage(Language language) {
	if(!contains(language)) {
	    Element element = language.getElement();
	    append(language);
	    Gatherer.c_man.configurationChanged();
	}
    }

    public void destroy() {
	if(controls != null) {
	    controls.destroy();
	    controls = null;
	}
	known_languages.clear();
	known_languages = null;
	default_language = null;
    }

    /** Method to retrieve the control for this manager.
     * @return the Control for editing the language partitions
     */
    public Control getControls() {
	if(controls == null) {
	    // Build controls
	    controls = new LanguageControl();
	}
	return controls;
    }

    /** Method to retrieve the default language code.
     * @return A <strong>Language</strong> containing a two letter code.
     */
    /* private Language getDefaultLanguage() {
	// If no default is set...
	if(default_language != null && default_language.isAssigned()) {
	    return default_language;
	}
	return null;
	} */

    /** Method to retrieve a certain language object by its code.
     * @param code The two letter code of a language, as a <strong>String</strong>.
     * @return The <strong>Language</strong> that matches the given code, or <i>null</i> if no such language exists.
     */
    public Language getLanguage(String code) {
	int size = getSize();
	for(int i = 0; i < size; i++) {
	    Language language = (Language) getElementAt(i);
	    if(language.getCode().equals(code)) {
		return language;
	    }
	}
	return null;
    }

    public ArrayList getLanguages() {
	return children();
    }

    /** Method to return a list of the known language codes.
     * @return an ArrayList containing the series of known language codes as per the languages.dat file
     */
    public ArrayList getLanguageCodes() {
	return new ArrayList(known_languages.keySet());
    }

    public String getLanguageName(String code) {
	return (String) known_languages.get(code);
    }

   private int moveLanguage(Language lang, boolean move_up)
    {
	// Determine the current position of the language
	int position = indexOf(lang);
	int new_position;

	// Attempt to move the language up
	if (move_up) {
	    // Check it's not already at the top
	    if (position == 0) {
		return position;
	    }

	    // This automatically removes the language first, as an Element can only exist once in a particular document
	    new_position = position - 1;
	    addBefore(lang, (Language) getElementAt(new_position));
	}

	// Attempt to move the language down
	else {
	    // Check it's not already at the bottom
	    if (position == (getSize()) - 1) {
		return position;
	    }

	    // This automatically removes the language first, as an Element can only exist once in a particular document
	    new_position = position + 1;
	    addAfter(lang, (Language) getElementAt(new_position));
	}

	// Schedule the collection for saving
	Gatherer.c_man.configurationChanged();
	return new_position;
    }

    /** Method to remove a certain language.
     * @param language The <strong>Language</strong> to remove.
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.collection.CollectionManager
     */
    private void removeLanguage(Language language) {
	remove(language);
	if(default_language != null && default_language.equals(language)) {
	    default_language = null;
	}
	Gatherer.c_man.configurationChanged();
    }

    /** Method to set the default language.
     * @param language The <strong>Language</strong> to use as a default, or <i>null</i> for no default.
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.collection.CollectionManager
     */
    public void setDefault(Language language) {
	if(language != null) {
	    if(default_language == null) {
		// Create the default index element, and place immediately after indexes element.
		Element default_language_element = root.getOwnerDocument().createElement(CollectionConfiguration.LANGUAGE_DEFAULT_ELEMENT);
		default_language = new Language(default_language_element);
		Node target_node = CollectionConfiguration.findInsertionPoint(default_language_element);
		if(target_node != null) {
		    root.getOwnerDocument().getDocumentElement().insertBefore(default_language_element, target_node);
		}
		else {
		    root.getOwnerDocument().getDocumentElement().appendChild(default_language_element);
		}
	    }
	    default_language.setAssigned(true);
	    default_language.setCode(language.getCode());
	}
	else {
	    if(default_language != null) {
		default_language.setAssigned(false);
	    }
	}
	Gatherer.c_man.configurationChanged();
    }


    /** This class represents the visual component of the Language Manager. */
    private class LanguageControl
	extends JPanel 
	implements Control {
	/** The button to add a new language support. */
	private JButton add_button = null;
	/** The button to remove a supported language. */
	private JButton remove_button = null;
	/** button to move a language up in the list */
	private JButton move_down_button;
	/** button to move a language down in the list */
	private JButton move_up_button;
	/** The button to set the current language as the default one. */
	private JButton set_default_button = null;
	/** A combobox listing the available supported languages. */
	private JComboBox language_combobox = null;
	/** A list of currently supported languages. */
	private JList selected_languages_list = null;
	/** Constructor.
	 * @see org.greenstone.gatherer.cdm.LanguageManager.LanguageControl.AddListener
	 * @see org.greenstone.gatherer.cdm.LanguageManager.LanguageControl.ClearDefaultListener
	 * @see org.greenstone.gatherer.cdm.LanguageManager.LanguageControl.ListListener
	 * @see org.greenstone.gatherer.cdm.LanguageManager.LanguageControl.RemoveListener
	 * @see org.greenstone.gatherer.cdm.LanguageManager.LanguageControl.SelectorListener
	 * @see org.greenstone.gatherer.cdm.LanguageManager.LanguageControl.SetDefaultListener
	 */
	public LanguageControl() {
	    super();
	    // Creation.
	    JPanel center_panel = new JPanel();

	    JLabel selected_languages_list_label = new JLabel();
	    Dictionary.registerText(selected_languages_list_label, "CDM.LanguageManager.Assigned_Languages");
	    selected_languages_list = new JList(model);
	    selected_languages_list.setCellRenderer(new MyLanguageListCellRenderer());
	    selected_languages_list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	    selected_languages_list.setVisibleRowCount(5);
	    JPanel details_panel = new JPanel();

	    JPanel control_panel = new JPanel();
	    JLabel selector_label = new JLabel();
	    Dictionary.registerText(selector_label, "CDM.LanguageManager.Selector");

	    language_combobox = new JComboBox(getLanguageCodes().toArray());
	    language_combobox.setPreferredSize(COMPONENT_SIZE);
	    language_combobox.setBackground(Configuration.getColor("coloring.collection_tree_background", false));
	    language_combobox.setRenderer(new LanguageListCellRenderer());
	    Dictionary.registerTooltip(language_combobox, "CDM.LanguageManager.Selector_Tooltip");

	    JPanel movement_pane = new JPanel();
	    move_up_button = new JButton("", JarTools.getImage("arrow-up.gif"));
	    move_up_button.setEnabled(false);
	    move_up_button.setMnemonic(KeyEvent.VK_U);
	    Dictionary.registerBoth(move_up_button, "CDM.Move.Move_Up", "CDM.Move.Move_Up_Tooltip");

	    move_down_button = new JButton("", JarTools.getImage("arrow-down.gif"));
	    move_down_button.setEnabled(false);
	    move_down_button.setMnemonic(KeyEvent.VK_D);
	    Dictionary.registerBoth(move_down_button, "CDM.Move.Move_Down", "CDM.Move.Move_Down_Tooltip");

	    set_default_button = new GLIButton();
	    set_default_button.setEnabled(false);
	    set_default_button.setMnemonic(KeyEvent.VK_S);
	    Dictionary.registerBoth(set_default_button, "CDM.LanguageManager.Set_Default", "CDM.LanguageManager.Set_Default_Tooltip");

	    JPanel button_panel = new JPanel();

	    add_button = new GLIButton();
	    add_button.setMnemonic(KeyEvent.VK_A);
	    Dictionary.registerBoth(add_button, "CDM.LanguageManager.Add", "CDM.LanguageManager.Add_Tooltip");
	    if (getLanguage((String)language_combobox.getItemAt(0))==null) {
		add_button.setEnabled(true);
	    } else {
		add_button.setEnabled(false);
	    }
	    remove_button = new GLIButton();
	    remove_button.setMnemonic(KeyEvent.VK_R);
	    remove_button.setEnabled(false);
	    Dictionary.registerBoth(remove_button, "CDM.LanguageManager.Remove", "CDM.LanguageManager.Remove_Tooltip");

	    // Set up and connect listeners.
	    add_button.addActionListener(new AddListener());
	    add_button.addActionListener(CollectionDesignManager.buildcol_change_listener);
	    move_down_button.addActionListener(new MoveListener(false));
	    move_down_button.addActionListener(CollectionDesignManager.buildcol_change_listener);
	    move_up_button.addActionListener(new MoveListener(true));
	    move_up_button.addActionListener(CollectionDesignManager.buildcol_change_listener);
	    remove_button.addActionListener(new RemoveListener());
	    remove_button.addActionListener(CollectionDesignManager.buildcol_change_listener);
	    language_combobox.addActionListener(new SelectorListener());
	    set_default_button.addActionListener(new SetDefaultListener());
	    set_default_button.addActionListener(CollectionDesignManager.buildcol_change_listener);
	    selected_languages_list.addListSelectionListener(new ListListener());

	    // Layout components
	    movement_pane.setBorder(BorderFactory.createEmptyBorder(0,2,0,0));
	    movement_pane.setLayout(new GridLayout(3,1));
	    movement_pane.add(move_up_button);
	    movement_pane.add(move_down_button);
	    movement_pane.add(set_default_button);

	    control_panel.setBorder(BorderFactory.createEmptyBorder(5,0,0,0));
	    control_panel.setLayout(new BorderLayout());
	    control_panel.add(selector_label, BorderLayout.WEST);
	    control_panel.add(language_combobox, BorderLayout.CENTER);

	    center_panel.setLayout(new BorderLayout());
	    center_panel.add(selected_languages_list_label, BorderLayout.NORTH);
	    center_panel.add(new JScrollPane(selected_languages_list), BorderLayout.CENTER);
	    center_panel.add(movement_pane, BorderLayout.EAST);
	    center_panel.add(control_panel, BorderLayout.SOUTH);

	    button_panel.setLayout(new GridLayout(1,2));
	    button_panel.add(add_button);
	    button_panel.add(remove_button);
// 	    button_panel.add(set_button);
// 	    button_panel.add(clear_button);

	    //when we go to a multi list, remove inner_panel and empty_panel
	    JPanel empty_panel = new JPanel();
	    JPanel inner_panel = new JPanel();

	    inner_panel.setLayout(new BorderLayout());
	    inner_panel.add(center_panel, BorderLayout.CENTER);
	    inner_panel.add(button_panel, BorderLayout.SOUTH);
	    setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	    setLayout(new BorderLayout());
	    //add(center_panel, BorderLayout.CENTER);
	    //add(button_panel, BorderLayout.SOUTH);
	    add(inner_panel, BorderLayout.NORTH);
	    add(empty_panel, BorderLayout.CENTER);
	}
	
	/** Destructor.	*/
	public void destroy() {
	}

	public void gainFocus() {
	}

	public void loseFocus() {
	}

	/** Listens for actions apon the 'add' button in the LanguageManager controls, and if detected calls the add method of the manager with a newly created language. */
	private class AddListener
	    implements ActionListener {
	    /** Add a new language support.
	     * @param event an ActionEvent
	     * @see org.greenstone.gatherer.cdm.Language
	     */
	    public void actionPerformed(ActionEvent event) {
		String language_code = (String) language_combobox.getSelectedItem();
		if(language_code != null) {
		    addLanguage(new Language(language_code));
		}
		// add some collectionmeta
		String lang_name = getLanguageName(language_code);
		CollectionMeta metadatum = new CollectionMeta("." + language_code);
		metadatum.setValue(lang_name);
		CollectionDesignManager.collectionmeta_manager.addMetadatum(metadatum);

		add_button.setEnabled(false);
	    }
	}

	/** Listens for actions apon the 'remove' button in the LanguageManager controls, and if detected calls the remove method of the manager with the language selected for removal. */
	private class RemoveListener
	    implements ActionListener {
	    /** Remove the currently selected language, if any.
	     * @param event An <strong>ActionEvent</strong>.
	     * @see org.greenstone.gatherer.cdm.Language
	     */
	    public void actionPerformed(ActionEvent event) {
		Language delete_me = (Language)selected_languages_list.getSelectedValue();
		if(delete_me != null) {
		    // Remove any current metadata for this language
		    CollectionDesignManager.collectionmeta_manager.removeMetadata("." + delete_me.getCode());
		    removeLanguage(delete_me);
		    if(default_language != null && default_language.equals(delete_me)) {
			setDefault(null);
			//clear_button.setEnabled(false);
			//default_language_field.setText("");
		    }
		}
		remove_button.setEnabled(false);
	    }
	}

	/** Listens for selections within the combobox on the LanguageManager controls, and if a change is detected enables, or disables, controls appropriately. */
	private class SelectorListener
	    implements ActionListener {
	    /** Enable or disable controls depeding on selection.
	     * @param event An <strong>ActionEvent</strong>.
	     */
	    public void actionPerformed(ActionEvent event) {
		String language_code = (String) language_combobox.getSelectedItem();
		if(language_code != null && getLanguage(language_code) == null) {
		    add_button.setEnabled(true);
		}
		else {
		    add_button.setEnabled(false);
		}
	    }
	}

	/** Listens for actions apon the 'set default' button in the LanguageManager controls, and if detected calls the <i>setDefault()</i> method of the manager with the language selected for default. */
	private class SetDefaultListener
	    implements ActionListener {
	    /** Set the default index to the one currently selected, if any.
	     * @param event An <strong>ActionEvent</strong>.
	     * @see org.greenstone.gatherer.cdm.Language
	     */
	    public void actionPerformed(ActionEvent event) {
		Language selected_language = (Language) selected_languages_list.getSelectedValue();
		if(selected_language != null) {
		    setDefault(selected_language);
		    //clear_button.setEnabled(true);
		    // This should cause a repaint of just the desired row
		    selected_languages_list.setSelectedValue(selected_language, true);
		}
		set_default_button.setEnabled(false);
	    }
	}

	private class MoveListener
	    implements ActionListener
	{
	    private boolean move_up;

	    public MoveListener(boolean move_up)
	    {
		this.move_up = move_up;
	    }

	    public void actionPerformed(ActionEvent event)
	    {
		// Retrieve the selected language
		Language language = (Language) selected_languages_list.getSelectedValue();
		if (language != null) {
		    int new_position = moveLanguage(language, move_up);
		    // Ensure the language that moved is still selected
		    selected_languages_list.setSelectedIndex(new_position);
		}
	    }
	}


	/** Listens for selections within the list on the LanguageManager controls, and if a change is detected enables, or disables, controls appropriately. */
	private class ListListener
	    implements ListSelectionListener {
	    /** Enable or disable controls depending on the current list selection.
	     * @param event A <strong>ListSelectionEvent</strong>.
	     */
	    public void valueChanged(ListSelectionEvent event) {
		if (event.getValueIsAdjusting()) {
		    return;
		}
		if(selected_languages_list.isSelectionEmpty()) {
		    remove_button.setEnabled(false);
		    set_default_button.setEnabled(false);
		    move_down_button.setEnabled(false);
 		    move_up_button.setEnabled(false);
		    return;
		}
		
		int i = selected_languages_list.getSelectedIndex();
 		int size = selected_languages_list.getModel().getSize();
		Language selected_lang = (Language)selected_languages_list.getSelectedValue();
		remove_button.setEnabled(true);
		set_default_button.setEnabled(default_language == null || !default_language.equals(selected_lang));
		
		if (i > 0) {
 		    move_up_button.setEnabled(true);
 		}
 		else {
 		    move_up_button.setEnabled(false);
 		}
 		if (i < size-1){
 		    move_down_button.setEnabled(true);
 		}
 		else {
 		    move_down_button.setEnabled(false);
 		}
	    }
	}

	private class MyLanguageListCellRenderer
	    extends DefaultListCellRenderer
	{
	    /** Return a component that has been configured to display the specified value. */
	    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
		JLabel component = (JLabel) super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
		if (default_language != null && default_language.equals(value)) {
		    component.setText(component.getText() + " [" + Dictionary.get("CDM.LanguageManager.Default_Language")+"]");
		}
		return component;
	    }
	}
    }
}
